/* eslint-disable guard-for-in */
/*
 * Esimerkki oman event emitterin luomisesta events.EventEmitter -luokasta
 *
 * Watcher tarkkailee mitä tiedostoja kansioon laitetaan ja heti kun kansioon
 * laitetaan tiedostoja, se muuntaa filujen nimet lowercaseen ja
 * siirtää muunnetut toiseen kansioon.
 *
 * Watcher perii eventEmitterin ja siitä luotu olio toimii event emitterinä
 *
 *  Eventtien avulla käsitellään yleensä tapahtumia jotka voivat tapahtua useita
 *  kertoja peräkkäin. Esimerkiksi tässä esimerkissä voidaan laittaa tiedostoja
 *  useita kertoja peräkkäin tarkkailtavaan kansioon
 */
const events = require('events'); //events -modulin käyttöönotto
const fs = require('fs'); //filesystem -modulin käyttöönotto

// Watcher -luokka
class Watcher extends events.EventEmitter {
  constructor(watchDir, processedDir) {
    super(); // yliluokan konstruktoria pitää muistaa kutsua
    this.watchDir = watchDir; //kansio jota tarkkaillaan
    this.processedDir = processedDir; //kansio jonne muunnetut laitetaan

    //console.log(this.watchDir); //katsotaan että kansioiden polut ovat ok
    //console.log(this.processedDir);

    console.log('Filewatcher käynnistetty ja tarkkailee tilannetta!');
  }

  watch() {
    fs.readdir(this.watchDir, function (err, files) {
      if (err) {
        throw err;
      }
      for (const index in files) {
        console.log('Tiedosto on laitettu watchdir -kansioon!');
        //process-eventin emitointi tapahtuu tässä
        watcher.emit('process', files[index]);
      }
    });
  }

  //käynnistysfunktio
  start() {
    //var watcher = this;
    fs.watchFile(watchDir, function () {
      watcher.watch();
    });
  }
}

const watchDir = __dirname + '\\watchdir'; //kansio jonne muunnettavat filut laitetaan
const processedDir = __dirname + '\\processeddir'; //kansio jonne muunnetut filut menevät

//watcher on uusi Watcher-olio ja event emitter koska on perinyt events.EventEmitter:in
const watcher = new Watcher(watchDir, processedDir);

/*watcher.on tarkoittaa että watcher tarkkailee process-eventtejä koko ajan.
  process on eventin nimi. function process(file) on event listener.
  Se on callback joka prosessoi filut eli muuttaa nimet lowercaseen
  ja laittaa ne processedDir -kansioon.
*/
watcher.on('process', function process(file) {
  const watchFile = this.watchDir + '/' + file;
  const processedFile = this.processedDir + '/' + file.toLowerCase();

  console.log('Tiedoston nimi on muunnettu lowercaseen!');
  console.log('Valmis tiedosto on laitettu processeddir -kansioon!');

  fs.rename(watchFile, processedFile, function (err) {
    if (err) {
      throw err;
    }
  });
});

watcher.start(); //filewatcherin käynnistys
